<?php
/**
 * Copyright © 2016 Filoblu S.r.l. All rights reserved.
 */
namespace FiloBlu\Esb\Observer;
use \Magento\Sales\Api\Data\OrderInterface;
use \FiloBlu\Esb\Helper\Data;
use \Magento\Framework\Event\Observer;
use \Magento\Framework\Event\ObserverInterface;

/**
 * Observer on Order Status Update
 */
 
 class CreateShipment implements ObserverInterface{
	
    /**
     * @var \FiloBlu\Esb\Model\ScheduleFactory
     */  	
    protected $_scheduleFactory;
    
	/**
     * @var \Psr\Log\LoggerInterface $logger
     */  
    protected $_logger;

	/**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */  
    protected $_scopeConfig;

	/**
     * @var \FiloBlu\Esb\Helper\Data
     */  
    protected $_helper;

        /** 
     * @var \Magento\Framework\Serialize\SerializerInterface 
     */
    protected $_serializer;
        
	/**
     * @param \FiloBlu\Esb\Model\ScheduleFactory $scheduleFactory
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \FiloBlu\Esb\Helper\Data $helper
     */  
	public function __construct(
            \FiloBlu\Esb\Model\ScheduleFactory $scheduleFactory, 
            \Psr\Log\LoggerInterface $logger,
            \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
            \FiloBlu\Esb\Helper\Data $helper,
            \Magento\Framework\Serialize\SerializerInterface $serializer        
	){
            $this->_scheduleFactory=$scheduleFactory;
            $this->_logger = $logger;
            $this->_scopeConfig = $scopeConfig;
            $this->_helper = $helper;
            $this->_serializer = $serializer;
	}

    /**
     * Send data to ESB schedule after invoice register. 
     *
     * @param \Magento\Framework\Event\Observer $observer
     * @return \Magento\Sales\Api\Data\OrderInterface 
     */
    public function execute(Observer $observer){      
        $shipment = $observer->getEvent()->getShipment();

        $events=$this->_serializer->unserialize($this->_scopeConfig->getValue(
            'esb/endpoints/events', 
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $shipment->getStoreId()
        ));  


        if($shipment->getOrigData("entity_id")==null && $events!=null && count($events)>0){
            foreach($events as $eventData){
                if($eventData["event_type"]!="create_shipment"){continue;}
                $endpoint=$this->_helper->getEndpointUrl($eventData["event_endpoint"]);
                $schedule = $this->_scheduleFactory->create();
                $eventDescription="create_shipment";
                $priority=($eventData["priority"])?$eventData["priority"]:0;                
                $scheduleData=array(
                        "object_repository_interface" => "\Magento\Sales\Api\ShipmentRepositoryInterface",
                        "object_id" => $shipment->getId(),
                        "endpoint" => $endpoint,
                        "event" => $eventDescription,
                        "sendmessage_class" => "\FiloBlu\Esb\Cron\SendMessage",
                        "status" => "pending",
                        "priority" => $priority,
                        "executable_after" => new \Zend_Db_Expr('NOW()')
                    );
                if(strlen($eventData["final_status"])>0){
                    $scheduleData["object_after_method"]="updateOrderStatus";
                    $scheduleData["object_after_method_argument"]=$eventData["final_status"];
                }
                $schedule->setData(
                    $scheduleData
                );
                $schedule->save();            
            }
        }
        return $shipment;
    }
}